-------------------------------------------------------------------------------
-- MrTrader 
--
-- Lightweight addon to access profession information of alts.
--
-- Send suggestions, comments, and bugs to user@zedonline.net
-------------------------------------------------------------------------------

local L = LibStub("AceLocale-3.0"):GetLocale("MrTrader", true);

-- Options for Configuration in the DB
local Options = {
	type = "group",
	get = function(item) return MrTrader.db.profile[item[#item]] end,
	set = function(item, value) MrTrader.db.profile[item[#item]] = value end,
	plugins = {},
	args = {
		menuOptions  = {
			type = "group",
			name = "Menu Options",
			order = 10,
			args = {
				showOppositeFaction = {
					type = "toggle",
					name = L["Show Opposing Faction Alts"],
					desc = L["Will show your alts from the opposing faction in the menu."],
					width = "full",
					order = 11,
				},
				showSecondarySkills = {
					type = "toggle",
					name = L["Show Minor Skills for Alts"],
					desc = L["Will show Cooking, Smelting, Runeforging, and First Aid on other characters."],
					width = "full",
					order = 12,
				},
			},
		},
		
		commandOptions = {
			type = "group",
			name = "Command Options",
			order = 20,
			args = {
				linkCommandType = {
					type = "group",
					name = "Listen to Commands From...",
					guiInline = true,
					order = 10,
					args = {
						respondToWhispers = {
							type = "toggle",
							name = "Whispers",
							desc = L["Will respond to whispers sent to you asking for skill links."],
							order = 21,
						},
						respondToGuild = {
							type = "toggle",
							name = "Guild Chat",
							desc = L["Will respond to guild messages asking for skill links."],
							order = 22,
						},
						respondToRaid = {
							type = "toggle",
							name = "Raid and Party Chat",
							desc = L["Will respond to raid and party messages asking for skill links."],
							order = 23,
						},
					},
				},

				linkPrivacyOptions = {
					type = "group",
					name = "Privacy Options",
					guiInline = true,
					order = 30,
					args = {
						whisperAltSkills = {
							type = "toggle",
							name = L["Whisper Skills for Alts"],
							desc = L["Will search your alts for skill links when responding to whisper commands."],
							width = "full",
							order = 31,
						},
						--includePlayerNames = {
						--	type = "toggle",
						--	name = "Include Character Names",
						--	desc = "Will show the name of the character with the skill being linked in whispered responses.",
						--	width = "full",
						--	order = 32,
						--}
					},
				},
				
				--linkCommandOptions = {
				--	type = "group",
				--	name = "Command Options",
				--	guiInline = true,
				--	order = 30,
				--	args = {
				--		showAllFullLinks = {
				--			type = "toggle",
				--			name = "Show All Full Links",
				--			desc = "Will show all matching links in response to !link/!full, instead of the highest matching.",
				--			order = 11,
				--		}
				--	},
				--},
			},
		},
	}
}

-- Defaults for options
local Defaults = {
	profile = {
		showOppositeFaction = true,
		showSecondarySkills = true,
		respondToWhispers = true,
		respondToGuild = false,
		respondToRaid = false,
		whisperAltSkills = false,
		includePlayerNames = false,
		showAllFullLinks = true,
		customTradeskillWindow = false,
	},
	realm = {
		skills = {},
	}
}

local ConsoleOptions = {
	type = "group",
	handler = MrTrader,
	args = {
		config = {
			type = "execute",
			name = L["Configure"];
			desc = L["Open configuration dialog"],
			func = function()
				InterfaceOptionsFrame_OpenToCategory(MrTrader.optionFrames.main);
			end
		},
	},
}

local ConsoleCmds = {
	"mrt",
	"mrtrader",
}

-- List of tradeskills
-- Key: Spell ID
-- Value: true if this skill has a tradeskill link you can share
local TradeSkills = {
	[2259] = { linkable = true, primary = true }, -- Alchemy
	[2018] = { linkable = true, primary = true }, -- Blacksmithing
	[2550] = { linkable = true, primary = false }, -- Cooking
	[7411] = { linkable = true, primary = true }, -- Enchanting
	[4036] = { linkable = true, primary = true }, -- Engineering
	[3273] = { linkable = true, primary = false }, -- First Aid
	[45357] = { linkable = true, primary = true }, -- Inscription
	[25229] = { linkable = true, primary = true }, -- Jewelcrafting
	[2108] = { linkable = true, primary = true }, -- Leatherworking
	[53428] = { linkable = false, primary = false }, -- Runeforging
	[2656] = { linkable = false, primary = false }, -- Smelting
	[3908] = { linkable = true, primary = true }, -- Tailoring
}

function MrTrader:Load()
	self.tradeskills = TradeSkills;
	self.db = LibStub("AceDB-3.0"):New("MrTraderDB", Defaults, "Default");

	-- Convert from v0.1 to v0.3 DBs
	if self.db.factionrealm.skills then
		if self.db.realm.skills then
			self.db.realm.skills = self:TableJoin(self.db.realm.skills, self.db.factionrealm.skills);
		else
			self.db.realm.skills = self.db.factionrealm.skills;
		end

		self.db.factionrealm.skills = nil;

		for name, skillTable in pairs(self.db.realm.skills) do
			skillTable["faction"] = UnitFactionGroup("player");
			self.db.realm.skills[name] = skillTable;
		end
	end
	
	-- Set a DB version for easier upgrading post v0.3
	if self.db.global.version == nil then
		self.db.global.version = 2;
	end
	
	self:RegisterOptions();
end

function MrTrader:RegisterOptions()
--	local profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db);
	Options.plugins["profiles"] = { profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db) };
	
	local config = LibStub("AceConfig-3.0");
	config:RegisterOptionsTable("MrTrader", ConsoleOptions, ConsoleCmds);
	
	local registry = LibStub("AceConfigRegistry-3.0");
	registry:RegisterOptionsTable("MrTrader Options", Options);
--	registry:RegisterOptionsTable("MrTrader Profiles", profiles);
	
	local dialog = LibStub("AceConfigDialog-3.0");
	self.optionFrames = {
		main = dialog:AddToBlizOptions("MrTrader Options", L["MrTrader"]),
--		profiles = dialog:AddToBlizOptions("MrTrader Profiles", L["Profiles"], L["MrTrader"]),
	};
end

function MrTrader:QuerySkills()
	local currentPlayer = UnitName("player");
	local currentFaction = UnitFactionGroup("player");	
	local skillTable = {};
	local oldTable = {};
	local didGrabUpdate = true;
	
	if self.db.realm.skills[currentPlayer] then
		oldTable = self.db.realm.skills[currentPlayer];
	end

	for skillID, value in pairs(TradeSkills) do
		usesLink = value.linkable;
		skillName = GetSpellInfo(skillID);
		doesExist, tradeLink = GetSpellLink(skillName);
		
		if doesExist and tradeLink then
			skillTable[skillID] = tradeLink;
		elseif doesExist and oldTable[skillID] then
			skillTable[skillID] = oldTable[skillID];
			didGrabUpdate = false;
		elseif doesExist then
			skillTable[skillID] = false;
			if usesLink then
				didGrabUpdate = false;
			end
		end
	end
	
	if self:TableSize(skillTable) > 0 then
		-- Saving the new table, copy over values that we want to keep
		skillTable["ignored"] = oldTable["ignored"];
		skillTable["faction"] = currentFaction;	

		self.db.realm.skills[currentPlayer] = skillTable;
		return didGrabUpdate;
	else
		self.db.realm.skills[currentPlayer] = nil;
		return true;
	end
end

function MrTrader:GetSkillRank(skillLink)
	local _, _, currentRank, maxRank = strsplit(':', skillLink, 4);
	
	return tonumber(currentRank);
end

function MrTrader:MatchPartialCraftSkillName(partialName)
	local matchedSkillID = nil;
	local matchCount = 0;
	
	partialName = partialName:upper();

	for skillID, skillValue in pairs(TradeSkills) do
		skillCanCraft = skillValue.linkable;
		if skillCanCraft then
			skillName = GetSpellInfo(skillID);
			skillName = skillName:upper();
			start = string.find( skillName , partialName , 1, true );
			
			if start then 
				matchCount = matchCount + 1;
				matchedSkillID = skillID;
			end
		end
	end
	
	if matchCount > 1 then
		return "ambiguous";
	else
		return matchedSkillID;
	end
end

function MrTrader:GetBestSkillLink(skillID)
	local currentFaction = UnitFactionGroup("player");	
	local currentPlayer = UnitName("player");
	local link = nil;
		
	local currentTable = self.db.realm.skills[currentPlayer];
	
	if self.db.realm.skills[currentPlayer][skillID] ~= nil then
		return self.db.realm.skills[currentPlayer][skillID];
	end
		
	if self.db.profile.whisperAltSkills then
		local currentLink = nil;
		local currentRank = 0;
	
		for playerName, skillList in pairs(self.db.realm.skills) do
			-- Find out if we need to search this character, and if it is in the same faction
			local sameFaction = true;
			if skillList["faction"] and skillList["faction"] ~= currentFaction then
				sameFaction = false;
			end
			
			if sameFaction and skillList[skillID] then
				local skillRank = MrTrader:GetSkillRank(skillList[skillID]);
				if skillRank > currentRank then
					currentLink = skillList[skillID];
					currentRank = skillRank;
				end
			end
		end
		
		return currentLink;
	end
	
	return nil;
end

function MrTrader:GetSkillTable()
	return self.db.realm.skills;
end

function MrTrader:RemoveCharacter(name)
	self.db.realm.skills[name] = nil;
end

function MrTrader:IgnoreCharacter(name)
	self.db.realm.skills[name]["ignored"] = true;
end

function MrTrader:UnignoreCharacter(name)
	self.db.realm.skills[name]["ignored"] = nil;
end

function MrTrader:CharacterIsIgnored(name)
	return self.db.realm.skills[name]["ignored"];
end